/**
 * 
 * AiCiA - Android IRC Client
 * 
 * Copyright (C)2010 GORRY.
 * 
 */

package net.gorry.aicia;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.RemoteException;
import android.os.Vibrator;
import android.text.Editable;
import android.text.SpannableStringBuilder;
import android.text.method.KeyListener;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ScrollView;

/**
 * @author GORRY
 *
 */
public class Layout {
	private static final String TAG = "ActivityMain";
	private static final boolean V = false;
	private static final int WC = ViewGroup.LayoutParams.WRAP_CONTENT;
	private static final int FP = ViewGroup.LayoutParams.FILL_PARENT;
	private final ActivityMain me;
	private IIRCService iIRCService;

	private LogLinearLayout mMainLogLayout;
	private LogLinearLayout mSubLogLayout;
	private LinearLayout mLogLayout;
	private LinearLayout mCmdLayout;
	private Button mChannelButton;
	private Button mChPrevButton;
	private Button mChNextButton;
	private Button mUserButton;

	private int mBaseLayoutWidth;
	@SuppressWarnings("unused")
	private int mBaseLayoutHeight;
	private int mLogLayoutWidth;
	private int mLogLayoutHeight;

	private Boolean mNotRestore = true;

	private KeyListener mInputBoxDefaultListener;

	/** */
	public LinearLayout mBaseLayout;
	/** */
	public LogWindow mMainLogWindow;
	/** */
	public LogWindow mSubLogWindow;
	/** */
	public EditText mInputBox;

	private boolean mCancelLongClickView = false;

	/**
	 * コンストラクタ
	 * @param a アクティビティインスタンス
	 */
	Layout(final ActivityMain a) {
		me = a;
	}

	/**
	 * iIRCServiceの登録
	 * @param i iIRCService
	 */
	public void setIIRCService(final IIRCService i) {
		iIRCService = i;
	}

	/**
	 *  アプリの一時退避
	 * @param outState 退避先
	 */
	public void saveInstanceState(final Bundle outState) {
		if (V) Log.v(TAG, "saveInstanceState()");
		mMainLogWindow.saveInstanceState(outState);
		mSubLogWindow.saveInstanceState(outState);
		outState.putCharSequence("mInputBox", mInputBox.getText());
		outState.putInt("mInputBox_selStart", mInputBox.getSelectionStart());
		outState.putInt("mInputBox_selEnd", mInputBox.getSelectionEnd());
	}

	/**
	 * アプリの一時退避復元
	 * @param savedInstanceState 復元元
	 */
	public void restoreInstanceState(final Bundle savedInstanceState) {
		if (V) Log.v(TAG, "restoreInstanceState()");
		mMainLogWindow.restoreInstanceState(savedInstanceState);
		mSubLogWindow.restoreInstanceState(savedInstanceState);
		mInputBox.setText(savedInstanceState.getCharSequence("mInputBox"));
		mInputBox.setSelection(savedInstanceState.getInt("mInputBox_selStart"), savedInstanceState.getInt("mInputBox_selEnd"));
		mInputBox.requestFocus();
		mNotRestore = false;
	}

	/**
	 * 回転方向の設定
	 * @param isFirst 初期化のときtrue
	 * @return 回転方向が変わったらtrue
	 */
	public boolean setOrientation(final boolean isFirst) {
		if (V) Log.v(TAG, "setOrientation()");
		final boolean lastOrientation = SystemConfig.getOrientation();
		final boolean isLandscape = (me.getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE);
		switch (SystemConfig.rotateMode) {
		case 0:  // 常に画面の向きに合わせる
			SystemConfig.setOrientation(isLandscape);
			break;
		case 1:  // スタート時の画面の向きで固定
			if (isFirst) {
				SystemConfig.setOrientation(isLandscape);
			}
			break;
		case 2:  // 縦向き表示で固定
			SystemConfig.setOrientation(false);
			break;
		case 3:  // 横向き表示で固定
			SystemConfig.setOrientation(true);
			break;
		}
		final boolean nowOrientation = SystemConfig.getOrientation();
		return (lastOrientation != nowOrientation);
	}

	/**
	 * 回転モードの設定
	 */
	public void setRotateMode() {
		if (V) Log.v(TAG, "setRotateMode()");
		final boolean isLandscape = SystemConfig.getOrientation();
		switch (SystemConfig.rotateMode) {
		default:
		case 0:
			me.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED);
			break;
		case 1:
			if (isLandscape) {
				me.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
			} else {
				me.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
			}
			break;
		case 2:
			me.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
			break;
		case 3:
			me.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
			break;
		}
	}

	/**
	 * ベースレイアウトのアップデート
	 */
	public void updateBaseLayout() {
		if (V) Log.v(TAG, "updateBaseLayout()");
		mBaseLayout.updateViewLayout(mLogLayout, new LinearLayout.LayoutParams(FP, FP, 1));
	}

	/**
	 * ベースレイアウト作成
	 * @param isFirst 最初ならtrue
	 */
	public synchronized void baseLayout_Create(final boolean isFirst) {
		if (V) Log.v(TAG, "baseLayout_Create()");
		Editable input = null;
		int selStart = 0;
		int selEnd = 0;
		if (mBaseLayout != null) {
			if (mInputBox != null) {
				input = mInputBox.getText();
				selStart = mInputBox.getSelectionStart();
				selEnd = mInputBox.getSelectionEnd();
			}
			mBaseLayout.removeAllViews();
			logLayout_Create(mBaseLayout);
			cmdLayout_Create(mBaseLayout);
		} else {
			mBaseLayout = new LinearLayout(me) {
				@Override
				protected void onSizeChanged(final int w, final int h, final int oldw, final int oldh) {
					if (V) Log.v(TAG, "onSizeChanged() baseLayout");
					mBaseLayoutWidth = w;
					mBaseLayoutHeight = h;
				}
			};
			mBaseLayout.setOrientation(LinearLayout.VERTICAL);
			mBaseLayout.setLayoutParams(new LinearLayout.LayoutParams(FP, FP));
			logLayout_Create(mBaseLayout);
			cmdLayout_Create(mBaseLayout);
		}
		if (input != null) {
			mInputBox.setText(input);
			mInputBox.setSelection(selStart, selEnd);
			mInputBox.requestFocus();
		}
	}

	/**
	 * フリックでチャンネル移動可能なレイアウト
	 */
	public class LogLinearLayout extends LinearLayout {
		private int xOnTouch = 0;
		private int yOnTouch = 0;
		private boolean shift = false;
		private boolean shortFlickHorizontal = false;
		private boolean longFlickHorizontal = false;
		private boolean flickRight = false;
		private boolean flickLeft = false;

		/**
		 * コンストラクタ
		 * @param context コンテキスト
		 */
		public LogLinearLayout(final Context context) {
			super(context);
		}

		/*
		 * @see android.view.View#onTouchEvent(android.view.MotionEvent)
		 */
		@Override
		public boolean onTouchEvent(final MotionEvent event) {
			final boolean result = super.onTouchEvent(event);
			return subOnTouchEvent(event, result);
		}

		/**
		 * レイアウトのタッチイベント処理
		 * @param event タッチイベント
		 * @param result super()での返り値
		 * @return 返すべき値
		 */
		public boolean subOnTouchEvent(final MotionEvent event, final boolean result) {
			switch (event.getAction()) {
			case MotionEvent.ACTION_DOWN:
			{
				xOnTouch = (int)event.getX();
				yOnTouch = (int)event.getY();
				shift = false;
				shortFlickHorizontal = false;
				longFlickHorizontal = false;
				mCancelLongClickView = false;
				flickLeft = false;
				flickRight = false;
				if (V) Log.v(TAG, "ACTION_DOWN: X=" + xOnTouch + ",Y="  + yOnTouch);
				return true;
			}
			case MotionEvent.ACTION_OUTSIDE:
			{
				shift = true;
				if (V) Log.v(TAG, "ACTION_OUTSIDE: X=" + xOnTouch + ",Y="  + yOnTouch);
				return true;
			}
			case MotionEvent.ACTION_UP:
			{
				final int xOnRelease = (int)event.getX();
				final int yOnRelease = (int)event.getY();
				final int xMove = xOnRelease - xOnTouch;
				final int yMove = yOnRelease - yOnTouch;
				@SuppressWarnings("unused")
				final int smallWidth = (mLogLayoutWidth > mLogLayoutHeight) ? mLogLayoutWidth : mLogLayoutHeight;
				final int edge = event.getEdgeFlags();
				if (V) Log.v(TAG, "ACTION_UP: X=" + xOnRelease + ",Y=" + yOnRelease + ",W=" + xMove + ",H=" + yMove + ",EDGE=" + edge);
				if (Math.abs(xMove) > Math.abs(yMove)) {
					if (longFlickHorizontal) {
						boolean changed = false;
						if (flickRight) {
							changed = ActivityMain.doMain.doChangeNextChannel(-1, 1, false);
						} else if (flickLeft) {
							changed = ActivityMain.doMain.doChangeNextChannel(1, 1, false);
						}
						if (!changed) {
							ActivityMain.myShortToastShow(me.getString(R.string.activitymain_java_notupdated));
						}
						return true;
					}
					if (shortFlickHorizontal) {
						boolean changed = false;
						if (flickRight) {
							final int f = (shift ? 1 : 0);
							changed = ActivityMain.doMain.doChangeNextChannel(-1, f, false);
						} else if (flickLeft) {
							final int f = (shift ? 1 : 0);
							changed = ActivityMain.doMain.doChangeNextChannel(1, f, false);
						}
						if (!changed) {
							ActivityMain.myShortToastShow(me.getString(R.string.activitymain_java_notchanged));
						}
						return true;
					}
				}
				xOnTouch = xOnRelease;
				yOnTouch = yOnRelease;
				shift = false;
				shortFlickHorizontal = false;
				longFlickHorizontal = false;
				mCancelLongClickView = false;
				flickLeft = false;
				flickRight = false;
				return result;
			}
			case MotionEvent.ACTION_MOVE:
			{
				mCancelLongClickView = true;
				final int xOnMove = (int)event.getX();
				final int yOnMove = (int)event.getY();
				final int xMove = xOnMove - xOnTouch;
				final int yMove = yOnMove - yOnTouch;
				final int smallWidth = (mLogLayoutWidth > mLogLayoutHeight) ? mLogLayoutWidth : mLogLayoutHeight;
				final int edge = event.getEdgeFlags();
				if (V) Log.v(TAG, "ACTION_MOVE: X=" + xOnMove + ",Y=" + yOnMove + ",W=" + xMove + ",H=" + yMove + ",EDGE=" + edge);
				if (Math.abs(xMove) > Math.abs(yMove)) {
					if (!longFlickHorizontal && (Math.abs(xMove) >= smallWidth/2)) {
						longFlickHorizontal = true;
						if (xMove > 0) {
							flickRight = true;
						} else {
							flickLeft = true;
						}
						final Vibrator vib = (Vibrator)me.getSystemService(Context.VIBRATOR_SERVICE);
						if (shortFlickHorizontal) {
							shortFlickHorizontal = false;
							new Thread(new Runnable() {	public void run() {
								vib.vibrate(50);
							} }).start();
						} else {
							new Thread(new Runnable() {	public void run() {
								final long[] pat = {0,50,250,50};
								vib.vibrate(pat, -1);
							} }).start();
						}
					}
					if (!shortFlickHorizontal && !longFlickHorizontal && (Math.abs(xMove) >= smallWidth/8)) {
						shortFlickHorizontal = true;
						if (xMove > 0) {
							flickRight = true;
						} else {
							flickLeft = true;
						}
						new Thread(new Runnable() {	public void run() {
							final Vibrator vib = (Vibrator)me.getSystemService(Context.VIBRATOR_SERVICE);
							vib.vibrate(50);
						} }).start();
					}
				}
				return result;
			}
			}
			return result;
		}

	}

	/**
	 * ログウィンドウレイアウト作成
	 * @param parentLayout 親レイアウト
	 */
	public synchronized void logLayout_Create(final LinearLayout parentLayout) {
		if (V) Log.v(TAG, "logLayout_Create()");

		// logLayout
		mLogLayout = new LinearLayout(me) {
			@Override
			protected void onSizeChanged(final int w, final int h, final int oldw, final int oldh) {
				if (V) Log.v(TAG, "onSizeChanged() logLayout");
				mLogLayoutWidth = w;
				mLogLayoutHeight = h;

				// if (SystemConfig.isLandscape) {
				if (SystemConfig.now_horizontalMode) {
					int w1;
					switch (SystemConfig.now_subWindowMode) {
					case 0:
						w1 = 0;
						break;
					case 1:
						w1 = mLogLayoutWidth/3;
						break;
					default:
					case 2:
						w1 = mLogLayoutWidth/2;
						break;
					case 3:
						w1 = mLogLayoutWidth/3*2;
						break;
					}
					mSubLogWindow.setWidth(w1);
					mSubLogLayout.updateViewLayout(mSubLogWindow.getScrollView(), new LinearLayout.LayoutParams(w1, FP, 3));
					mLogLayout.updateViewLayout(mSubLogLayout, new LinearLayout.LayoutParams(w1, FP, 2));

					final int w2 = mLogLayoutWidth-w1;
					mMainLogWindow.setWidth(w2);
					mMainLogLayout.updateViewLayout(mMainLogWindow.getScrollView(), new LinearLayout.LayoutParams(w2, FP, 3));
					mLogLayout.updateViewLayout(mMainLogLayout, new LinearLayout.LayoutParams(w2, FP, 2));
				} else {
					int h1;
					switch (SystemConfig.now_subWindowMode) {
					case 0:
						h1 = 0;
						break;
					case 1:
						h1 = mLogLayoutHeight/3;
						break;
					default:
					case 2:
						h1 = mLogLayoutHeight/2;
						break;
					case 3:
						h1 = mLogLayoutHeight/3*2;
						break;
					}
					mSubLogWindow.setWidth(mLogLayoutWidth);
					mSubLogLayout.updateViewLayout(mSubLogWindow.getScrollView(), new LinearLayout.LayoutParams(FP, h1, 3));
					mLogLayout.updateViewLayout(mSubLogLayout, new LinearLayout.LayoutParams(FP, h1, 2));

					final int h2 = mLogLayoutHeight-h1;
					mMainLogWindow.setWidth(mLogLayoutWidth);
					mMainLogLayout.updateViewLayout(mMainLogWindow.getScrollView(), new LinearLayout.LayoutParams(FP, h2, 3));
					mLogLayout.updateViewLayout(mMainLogLayout, new LinearLayout.LayoutParams(FP, h2, 2));
				}
				mBaseLayout.updateViewLayout(mLogLayout, new LinearLayout.LayoutParams(mBaseLayoutWidth, FP, 1));
			}
		};

		// if (SystemConfig.isLandscape) {
		if (SystemConfig.now_horizontalMode) {
			mLogLayout.setOrientation(LinearLayout.HORIZONTAL);
		} else {
			mLogLayout.setOrientation(LinearLayout.VERTICAL);
		}
		mLogLayout.setLayoutParams(new LinearLayout.LayoutParams(FP, FP, 1));
		parentLayout.addView(mLogLayout);

		{
			// mainLogLayout
		}
		mMainLogLayout = new LogLinearLayout(me) {
			@Override
			protected void onSizeChanged(final int w, final int h, final int oldw, final int oldh) {
				if (V) Log.v(TAG, "onSizeChanged() mainLogLayout");
				mMainLogWindow.registerParentLayout(w, h);
			}
		};
		mMainLogLayout.setOrientation(LinearLayout.HORIZONTAL);
		mMainLogLayout.setLayoutParams(new LinearLayout.LayoutParams(FP, FP, 2));
		mMainLogLayout.setBackgroundColor(SystemConfig.mainLogBackgroundColor[SystemConfig.nowColorSet]);
		mLogLayout.addView(mMainLogLayout);
		mMainLogWindow = new LogWindow(me, "MainLog");
		mMainLogWindow.setTextSize(SystemConfig.now_mainLogFontSize);
		mMainLogWindow.setDateColor(SystemConfig.mainLogDateColor[SystemConfig.nowColorSet]);
		mMainLogWindow.setTextColorResource(R.color.textcolor_mainlog);
		mMainLogWindow.setLogBufferSize(SystemConfig.mainLogBufferSize);
		mMainLogWindow.createWindow(me, mMainLogLayout);
		mMainLogWindow.getTextView().setOnLongClickListener(onMainLogTextViewLongClick);
		mMainLogLayout.setOnLongClickListener(onMainLogTextViewLongClick);  // テキスト非表示部

		// subLogLayout
		mSubLogLayout = new LogLinearLayout(me) {
			@Override
			protected void onSizeChanged(final int w, final int h, final int oldw, final int oldh) {
				if (V) Log.v(TAG, "onSizeChanged() subLogLayout");
				mSubLogWindow.registerParentLayout(w, h);
			}
		};
		mSubLogLayout.setOrientation(LinearLayout.HORIZONTAL);
		mSubLogLayout.setLayoutParams(new LinearLayout.LayoutParams(FP, FP, 2));
		mSubLogLayout.setBackgroundColor(SystemConfig.subLogBackgroundColor[SystemConfig.nowColorSet]);
		mLogLayout.addView(mSubLogLayout);
		mSubLogWindow = new LogWindow(me, "SubLog");
		mSubLogWindow.setTextSize(SystemConfig.now_subLogFontSize);
		mSubLogWindow.setDateColor(SystemConfig.subLogDateColor[SystemConfig.nowColorSet]);
		mSubLogWindow.setTextColorResource(R.color.textcolor_sublog);
		mSubLogWindow.setLogBufferSize(SystemConfig.subLogBufferSize);
		mSubLogWindow.createWindow(me, mSubLogLayout);
		mSubLogWindow.getTextView().setOnLongClickListener(onSubLogTextViewLongClick);
		mSubLogLayout.setOnLongClickListener(onSubLogTextViewLongClick);  // テキスト非表示部
		if (SystemConfig.now_horizontalMode) {
			mSubLogWindow.getTextView().setPadding(SystemConfig.now_subLogFontSize/4, 0, 0, 0);
		}

		if ( mNotRestore ) {
			mMainLogWindow.addMessage("Main-log", SystemConfig.mainLogTextColor[SystemConfig.nowColorSet], IRCMsg.getDateMsg());
			mMainLogWindow.addMessage("Push [MENU] to start IRC.", SystemConfig.mainLogTextColor[SystemConfig.nowColorSet], IRCMsg.getDateMsg());
		}
	}


	/**
	 * コマンドラインレイアウト作成
	 * @param parentLayout 親レイアウト
	 */
	public synchronized void cmdLayout_Create(final LinearLayout parentLayout) {
		if (V) Log.v(TAG, "cmdLayout_Create()");

		// cmdLayout;
		mCmdLayout = new LinearLayout(me);
		mCmdLayout.setOrientation(LinearLayout.HORIZONTAL);
		mCmdLayout.setLayoutParams(new LinearLayout.LayoutParams(FP, WC));
		parentLayout.addView(mCmdLayout);

		{
			// inputBox;
			mInputBox = new EditText(me);
			mInputBox.setLayoutParams(new LinearLayout.LayoutParams(WC, WC, 1));
			if (SystemConfig.now_editFontSize > 0) {
				mInputBox.setTextSize(SystemConfig.now_editFontSize);
				// minputBox.setHeight(SystemConfig.now_editFontSize);
			}
			mInputBoxDefaultListener = mInputBox.getKeyListener();
			mInputBox.setKeyListener(onInputBoxKey);
			mInputBox.setFreezesText(true);
			mCmdLayout.addView(mInputBox);

			// ChPrevButton
			mChPrevButton = new Button(me);
			mChPrevButton.setLayoutParams(new LinearLayout.LayoutParams(WC, WC));
			mChPrevButton.setText("↑");
			if (SystemConfig.cmdLineTextSize > 0) {
				mChPrevButton.setTextSize(SystemConfig.cmdLineTextSize);
				// mChPrevButton.setHeight(SystemConfig.cmdLineTextSize);
			}
			mChPrevButton.setOnClickListener(onChPrevButtonClick);
			mChPrevButton.setOnLongClickListener(onChPrevButtonLongClick);
			mCmdLayout.addView(mChPrevButton);
			mChPrevButton.setKeyListener(onButtonKey);


			// channelButton
			mChannelButton = new Button(me);
			mChannelButton.setLayoutParams(new LinearLayout.LayoutParams(WC, WC));
			mChannelButton.setText("Ch");
			if (SystemConfig.cmdLineTextSize > 0) {
				mChannelButton.setTextSize(SystemConfig.cmdLineTextSize);
				// mChannelButton.setHeight(SystemConfig.cmdLineTextSize);
			}
			mChannelButton.setOnClickListener(onChannelButtonClick);
			mChannelButton.setOnLongClickListener(onChannelButtonLongClick);
			mCmdLayout.addView(mChannelButton);
			mChannelButton.setKeyListener(onButtonKey);

			// chNextButton
			mChNextButton = new Button(me);
			mChNextButton.setLayoutParams(new LinearLayout.LayoutParams(WC, WC));
			mChNextButton.setText("↓");
			if (SystemConfig.cmdLineTextSize > 0) {
				mChNextButton.setTextSize(SystemConfig.cmdLineTextSize);
				// mChNextButton.setHeight(SystemConfig.cmdLineTextSize);
			}
			mChNextButton.setOnClickListener(onChNextButtonClick);
			mChNextButton.setOnLongClickListener(onChNextButtonLongClick);
			mCmdLayout.addView(mChNextButton);
			mChNextButton.setKeyListener(onButtonKey);

			// userButton
			mUserButton = new Button(me);
			mUserButton.setLayoutParams(new LinearLayout.LayoutParams(WC, WC));
			mUserButton.setText("U");
			if (SystemConfig.cmdLineTextSize > 0) {
				mUserButton.setTextSize(SystemConfig.cmdLineTextSize);
				// mUserButton.setHeight(SystemConfig.cmdLineTextSize);
			}
			mUserButton.setOnClickListener(onUserButtonClick);
			mUserButton.setOnLongClickListener(onUserButtonLongClick);
			mCmdLayout.addView(mUserButton);
			mUserButton.setKeyListener(onButtonKey);
		}
	}

	/**
	 * 入力ボックスの保存
	 */
	public void saveInputBox() {
		if (V) Log.v(TAG, "saveInputBox()");
		if ((mInputBox != null) && (iIRCService != null)) {
			try {
				iIRCService.saveInputBox(
						mInputBox.getText().toString(),
						mInputBox.getSelectionStart(),
						mInputBox.getSelectionEnd()
				);
			} catch (final RemoteException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 入力ボックスの復元
	 */
	public void restoreInputBox() {
		if (V) Log.v(TAG, "restoreInputBox()");
		if (mInputBox != null) {
			if (mInputBox != null) {
				me.runOnUiThread(new Runnable(){ public void run() {
					try {
						mInputBox.setText(iIRCService.loadInputBox());
						final int selStart = iIRCService.loadInputBoxSelStart();
						final int selEnd = iIRCService.loadInputBoxSelEnd();
						mInputBox.setSelection(selStart, selEnd);
					} catch (final RemoteException e) {
						e.printStackTrace();
					}
					mInputBox.requestFocus();
				} });
			}
		}
	}

	/**
	 * 入力メッセージの送信
	 */
	private void sendMessageByInputBox() {
		if (V) Log.v(TAG, "sendMessageByInputBox()");
		final String dateMsg = IRCMsg.getDateMsg();
		try {
			final String serverName = ActivityMain.mCurrentServerName;
			final String channel = ActivityMain.mCurrentChannel;
			if ((serverName != null) && (channel != null)) {
				@SuppressWarnings("unused")
				final String nick = iIRCService.getNick(serverName);
				final String message = mInputBox.getText().toString();
				if (message.length() > 0) {
					if (message.startsWith("/")) {
						iIRCService.sendCommandLine(serverName, channel, message);
					} else {
						iIRCService.sendMessageToChannel(serverName, channel, message);
					}
				}
			}
		}
		catch(final RemoteException e) {
			mMainLogWindow.addMessage("** exception on sendMessageByInputBox()", SystemConfig.mainLogTextColor[SystemConfig.nowColorSet], dateMsg);
		}
		mInputBox.setText("");
	}

	/**
	 * 入力ボックスのリスナ処理
	 */
	private final KeyListener onInputBoxKey = new KeyListener() {
		public int getInputType() {
			return mInputBoxDefaultListener.getInputType();
		}

		public boolean onKeyDown(final View view, final Editable text, final int keyCode, final KeyEvent event) {
			if (keyCode == KeyEvent.KEYCODE_ENTER) {
				sendMessageByInputBox();
				return true;
			}
			final boolean k = ActivityMain.doMain.doShortcutKey(keyCode, event);
			if (k == true) return true;
			return mInputBoxDefaultListener.onKeyDown(view, text, keyCode, event);
		}

		public boolean onKeyUp(final View view, final Editable text, final int keyCode, final KeyEvent event) {
			return mInputBoxDefaultListener.onKeyUp(view, text, keyCode, event);
		}

		public boolean onKeyOther(final View view, final Editable text, final KeyEvent event) {
			return mInputBoxDefaultListener.onKeyOther(view, text, event);
		}

		public void clearMetaKeyState(final View view, final Editable content, final int states) {
			mInputBoxDefaultListener.clearMetaKeyState(view, content, states);
		}

	};

	/**
	 * サービスから復活データを貰う
	 */
	public synchronized void reviveLogs() {
		if (V) Log.v(TAG, "reviveLogs()");
		try {
			final SpannableStringBuilder subLog = new SpannableStringBuilder(iIRCService.getSpanChannelLog(IRCMsg.sOtherChannelName, null));
			if (SystemConfig.now_subWindowMode > 0) {
				mSubLogWindow.setMessage(subLog);
			}
			final String serverName = iIRCService.getCurrentServerName();
			if (serverName != null) {
				final String channel = iIRCService.getCurrentChannel(serverName);
				if (channel != null) {
					ActivityMain.doMain.doChangeChannel(serverName, channel, false);
					// final SpannableStringBuilder chLog = new SpannableStringBuilder(iIRCService.getSpanChannelLog(serverName, channel));
					// mMainLogWindow.setMessage(chLog);
				}
			}
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * レイアウトのコンフィギュレーション変更
	 * @param newConfig 新内容
	 */
	public synchronized void changeConfiguration(final Configuration newConfig) {
		if (V) Log.v(TAG, "onConfigurationChanged()");
		final boolean isChanged = setOrientation(false);
		if (isChanged) {
			setRotateMode();
			baseLayout_Create(false);
			if (iIRCService != null) reviveLogs();
		}
		mBaseLayout.requestLayout();
	}

	/**
	 * レイアウトの再作成処理
	 */
	public synchronized void rebootLayout() {
		if (V) Log.v(TAG, "rebootLayout()");
		final boolean isChanged = setOrientation(true);
		if (isChanged || (SystemConfig.rotateMode == 0)) {
			setRotateMode();
			baseLayout_Create(false);
			if (iIRCService != null) reviveLogs();
		}
	}

	/**
	 * MainLogTextViewのロングクリック処理
	 */
	private final OnLongClickListener onMainLogTextViewLongClick = new OnLongClickListener() {
		public boolean onLongClick(final View v) {
			if (V) Log.v(TAG, "onMainLogTextViewLongClick: onLongClick()");
			if (mCancelLongClickView) {
				mCancelLongClickView = false;
				return true;
			}

			// メインログからコピー
			final String text = mMainLogWindow.getTextView().getText().toString();
			final DialogInterface.OnClickListener MyClickListener = new DialogInterface.OnClickListener(){
				public void onClick(final DialogInterface dialog, final int which) {
					// 何もしない
				}
			};
			final AlertDialog.Builder bldr = new AlertDialog.Builder(me)
			.setTitle(R.string.activitymain_java_copyfrommainlog);
			bldr.setNegativeButton("Close", MyClickListener);
			final ScrollView scroll = new ScrollView(me);
			final EditText edit = new EditText(me);
			edit.setText(text);
			final EditAssist editAssist = new EditAssist(me);
			editAssist.setOldKeyListener(edit.getKeyListener());
			edit.setKeyListener(editAssist.newKeyListener);
			edit.setSelection(text.length());
			edit.setTextSize(SystemConfig.now_editFontSize);
			scroll.addView(edit);
			bldr.setView(scroll);
			bldr.create().show();
			return true;
		}
	};

	/**
	 * SubLogTextViewのロングクリック処理
	 */
	private final OnLongClickListener onSubLogTextViewLongClick = new OnLongClickListener() {
		public boolean onLongClick(final View v) {
			if (V) Log.v(TAG, "onSubLogTextViewLongClick: onLongClick()");
			if (mCancelLongClickView) {
				mCancelLongClickView = false;
				return true;
			}

			// サブログからコピー
			final String text = mSubLogWindow.getTextView().getText().toString();
			final DialogInterface.OnClickListener MyClickListener = new DialogInterface.OnClickListener(){
				public void onClick(final DialogInterface dialog, final int which) {
					// 何もしない
				}
			};
			final AlertDialog.Builder bldr = new AlertDialog.Builder(me)
			.setTitle(R.string.activitymain_java_copyfromsublog);
			bldr.setNegativeButton("Close", MyClickListener);
			final ScrollView scroll = new ScrollView(me);
			final EditText edit = new EditText(me);
			edit.setText(text);
			final EditAssist editAssist = new EditAssist(me);
			editAssist.setOldKeyListener(edit.getKeyListener());
			edit.setKeyListener(editAssist.newKeyListener);
			edit.setSelection(text.length());
			edit.setTextSize(SystemConfig.now_editFontSize);
			scroll.addView(edit);
			bldr.setView(scroll);
			bldr.create().show();
			return true;
		}
	};

	/**
	 * ボタンのキーリスナ処理
	 */
	private final KeyListener onButtonKey = new KeyListener() {
		public int getInputType() {
			return 0;
		}

		public boolean onKeyDown(final View view, final Editable text, final int keyCode, final KeyEvent event) {
			final boolean k = ActivityMain.doMain.doShortcutKey(keyCode, event);
			if (k == true) return true;
			if (EditAssist.isAnyKey(keyCode)) {
				ActivityMain.layout.mInputBox.requestFocus();
				return true;
			}
			return false;
		}

		public boolean onKeyUp(final View view, final Editable text, final int keyCode, final KeyEvent event) {
			return false;
		}

		public boolean onKeyOther(final View view, final Editable text, final KeyEvent event) {
			return false;
		}

		public void clearMetaKeyState(final View view, final Editable content, final int states) {
			return;
		}

	};


	/**
	 * [Ch]ボタンのクリック処理
	 */
	private final OnClickListener onChannelButtonClick = new OnClickListener() {
		public void onClick(final View v) {
			if (V) Log.v(TAG, "onChannelButtonClick: onLongClick()");
			ActivityMain.doMain.doSelectChannel();
		}
	};

	/**
	 * [Ch]ボタンのロングクリック処理
	 */
	private final OnLongClickListener onChannelButtonLongClick = new OnLongClickListener() {
		public boolean onLongClick(final View v) {
			if (V) Log.v(TAG, "onChannelButtonLongClick: onLongClick()");
			ActivityMain.doMain.doSelectUpdatedChannel();
			return true;
		}
	};

	/**
	 * [↑]ボタンのクリック処理
	 */
	private final OnClickListener onChPrevButtonClick = new OnClickListener() {
		public void onClick(final View v) {
			if (V) Log.v(TAG, "onChPrevButtonClick: onClick()");
			final boolean changed = ActivityMain.doMain.doChangeNextChannel(-1, 0, false);
			if (!changed) {
				ActivityMain.myShortToastShow(me.getString(R.string.activitymain_java_notchanged));
			}
		}
	};

	/**
	 * [↑]ボタンのロングクリック処理
	 */
	private final OnLongClickListener onChPrevButtonLongClick = new OnLongClickListener() {
		public boolean onLongClick(final View v) {
			if (V) Log.v(TAG, "onChPrevButtonLongClick: onLongClick()");
			final boolean updated = ActivityMain.doMain.doChangeNextChannel(-1, 1, false);
			if (!updated) {
				ActivityMain.myShortToastShow(me.getString(R.string.activitymain_java_notupdated));
			}
			return true;
		}
	};

	/**
	 * [↓]ボタンのクリック処理
	 */
	private final OnClickListener onChNextButtonClick = new OnClickListener() {
		public void onClick(final View v) {
			if (V) Log.v(TAG, "onChNextButtonClick: onClick()");
			final boolean changed = ActivityMain.doMain.doChangeNextChannel(1, 0, false);
			if (!changed) {
				ActivityMain.myShortToastShow(me.getString(R.string.activitymain_java_notchanged));
			}
		}
	};

	/**
	 * [↓]ボタンのロングクリック処理
	 */
	private final OnLongClickListener onChNextButtonLongClick = new OnLongClickListener() {
		public boolean onLongClick(final View v) {
			if (V) Log.v(TAG, "onChNextButtonLongClick: onLongClick()");
			final boolean updated = ActivityMain.doMain.doChangeNextChannel(1, 1, false);
			if (!updated) {
				ActivityMain.myShortToastShow(me.getString(R.string.activitymain_java_notupdated));
			}
			return true;
		}
	};

	/**
	 * [U]ボタンのクリック処理
	 */
	private final OnClickListener onUserButtonClick = new OnClickListener() {
		public void onClick(final View v) {
			if (V) Log.v(TAG, "onUserButtonClick: onClick()");
			ActivityMain.doMain.doUtility(0);
		}
	};

	/**
	 * [U]ボタンのロングクリック処理
	 */
	private final OnLongClickListener onUserButtonLongClick = new OnLongClickListener() {
		public boolean onLongClick(final View v) {
			if (V) Log.v(TAG, "onUserButtonLongClick: onLongClick()");
			if (ActivityMain.mCurrentServerName != null) {
				ActivityMain.doMain.doUtility(1);
			}
			return true;
		}
	};


}
